@@ -1,7 +1,8 @@
 // Copyright (C) 2014 Denis Shienkov <denis.shienkov@gmail.com>
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/QtTest>
+#include <QtTest/private/qpropertytesthelper_p.h>
 #include <QtSerialPort/QSerialPort>
 #include <QtSerialPort/QSerialPortInfo>
 
@@ -107,8 +108,13 @@ private slots:
     void readWriteWithDifferentBaudRate_data();
     void readWriteWithDifferentBaudRate();
 
+    void readWriteWithMarkSpaceParity_data();
+    void readWriteWithMarkSpaceParity();
+
     void bindingsAndProperties();
 
+    void readyReadNotRecursive();
+
 protected slots:
     void handleBytesWrittenAndExitLoopSlot(qint64 bytesWritten);
     void handleBytesWrittenAndExitLoopSlot2(qint64 bytesWritten);
@@ -860,14 +866,14 @@ class AsyncWriterByTimer : public QObject
 {
     Q_OBJECT
 public:
-    explicit AsyncWriterByTimer(
-            QSerialPort &port, Qt::ConnectionType connectionType, const QByteArray &dataToWrite)
-        : serialPort(port), writeChunkSize(0)
+    explicit AsyncWriterByTimer(QSerialPort &port, Qt::ConnectionType connectionType,
+                                const QByteArray &dataToWrite, int interval = 0)
+        : serialPort(port), writeChunkSize(0), timerMsec(interval)
     {
         writeBuffer.setData(dataToWrite);
         writeBuffer.open(QIODevice::ReadOnly);
         connect(&timer, &QTimer::timeout, this, &AsyncWriterByTimer::send, connectionType);
-        timer.start(0);
+        timer.start(timerMsec);
     }
 
 private slots:
@@ -884,6 +890,7 @@ private:
     QBuffer writeBuffer;
     int writeChunkSize;
     QTimer timer;
+    int timerMsec;
 };
 
 void tst_QSerialPort::asynchronousWriteByTimer_data()
@@ -1261,148 +1268,210 @@ void tst_QSerialPort::readWriteWithDifferentBaudRate()
     }
 }
 
+void tst_QSerialPort::readWriteWithMarkSpaceParity_data()
+{
+    QTest::addColumn<QSerialPort::Parity>("parity");
+
+    QTest::newRow("parity_space") << QSerialPort::SpaceParity;
+    QTest::newRow("parity_mark") << QSerialPort::MarkParity;
+}
+
+void tst_QSerialPort::readWriteWithMarkSpaceParity()
+{
+    QFETCH(const QSerialPort::Parity, parity);
+
+    auto setupPort = [&parity](QSerialPort &port) {
+        port.setBaudRate(QSerialPort::Baud9600);
+        port.setParity(parity);
+        port.setDataBits(QSerialPort::Data8);
+        port.setStopBits(QSerialPort::OneStop);
+    };
+
+    QSerialPort sender(m_senderPortName);
+    QSignalSpy senderSpy(&sender, &QSerialPort::bytesWritten);
+    setupPort(sender);
+    QVERIFY(sender.open(QIODevice::ReadWrite));
+
+    QSerialPort receiver(m_receiverPortName);
+    QSignalSpy receiverSpy(&receiver, &QSerialPort::readyRead);
+    setupPort(receiver);
+    QVERIFY(receiver.open(QIODevice::ReadWrite));
+
+    const QByteArray data("some data");
+    const qint64 written = sender.write(data);
+    QCOMPARE(written, qint64(data.size()));
+    QTRY_COMPARE(senderSpy.size(), 1);
+    QCOMPARE(senderSpy.at(0).at(0).value<qint64>(), written);
+
+    QTRY_COMPARE_GE(receiver.bytesAvailable(), written);
+    QCOMPARE_GE(receiverSpy.size(), 1); // we should get *at least one* signal
+
+#if !defined(Q_OS_UNIX) || defined(CMSPAR)
+    // On UNIX with no CMSPAR the result is flaky, so we cannot even use
+    // QEXPECT_FAIL(). See QTBUG-131679.
+    const QByteArray receivedData = receiver.readAll();
+    QCOMPARE(receivedData, data);
+#endif
+}
+
 void tst_QSerialPort::bindingsAndProperties()
 {
     QSerialPort sp;
 
     // -- data bits
 
-    QProperty<QSerialPort::DataBits> dbProp(QSerialPort::DataBits::Data6);
-    QCOMPARE(dbProp.value(), QSerialPort::DataBits::Data6);
-
-    sp.bindableDataBits().setBinding(Qt::makePropertyBinding(dbProp));
-    QCOMPARE(sp.dataBits(), QSerialPort::DataBits::Data6);
-
-    const QSignalSpy dataBitsChangedSpy(&sp, &QSerialPort::dataBitsChanged);
-
-    dbProp = QSerialPort::DataBits::Data5;
-    QCOMPARE(sp.dataBits(), QSerialPort::DataBits::Data5);
-    QCOMPARE(dataBitsChangedSpy.size(), 1);
-    QCOMPARE(dataBitsChangedSpy.at(0).value(0).toInt(), QSerialPort::DataBits::Data5);
-
-    dbProp.setBinding(sp.bindableDataBits().makeBinding());
-    sp.setDataBits(QSerialPort::DataBits::Data8);
-    QCOMPARE(dbProp.value(), QSerialPort::DataBits::Data8);
-
-    dbProp.setBinding([&] { return sp.dataBits(); });
-    sp.setDataBits(QSerialPort::DataBits::Data7);
-    QCOMPARE(dbProp.value(), QSerialPort::DataBits::Data7);
+    QTestPrivate::testReadWritePropertyBasics(sp, QSerialPort::DataBits::Data6,
+                                              QSerialPort::DataBits::Data5, "dataBits");
+    if (QTest::currentTestFailed()) {
+        qDebug("Failed property test for QSetialPort::dataBits");
+        return;
+    }
 
     // -- parity
 
-    QProperty<QSerialPort::Parity> parityProp(QSerialPort::Parity::SpaceParity);
-    QCOMPARE(parityProp.value(), QSerialPort::Parity::SpaceParity);
-
-    sp.bindableParity().setBinding(Qt::makePropertyBinding(parityProp));
-    QCOMPARE(sp.parity(), QSerialPort::Parity::SpaceParity);
-
-    const QSignalSpy parityChangedSpy(&sp, &QSerialPort::parityChanged);
-
-    parityProp = QSerialPort::Parity::EvenParity;
-    QCOMPARE(sp.parity(), QSerialPort::Parity::EvenParity);
-    QCOMPARE(parityChangedSpy.size(), 1);
-    QCOMPARE(parityChangedSpy.at(0).value(0).toInt(), QSerialPort::Parity::EvenParity);
-
-    parityProp.setBinding(sp.bindableParity().makeBinding());
-    sp.setParity(QSerialPort::Parity::NoParity);
-    QCOMPARE(parityProp.value(), QSerialPort::Parity::NoParity);
-
-    parityProp.setBinding([&] { return sp.parity(); });
-    sp.setParity(QSerialPort::Parity::OddParity);
-    QCOMPARE(parityProp.value(), QSerialPort::Parity::OddParity);
+    QTestPrivate::testReadWritePropertyBasics(sp, QSerialPort::Parity::SpaceParity,
+                                              QSerialPort::Parity::EvenParity, "parity");
+    if (QTest::currentTestFailed()) {
+        qDebug("Failed property test for QSetialPort::parity");
+        return;
+    }
 
     // -- stop bits
 
-    QProperty<QSerialPort::StopBits> sbProp(QSerialPort::StopBits::OneAndHalfStop);
-    QCOMPARE(sbProp.value(), QSerialPort::StopBits::OneAndHalfStop);
+    static_assert(std::is_same_v<decltype(sp.bindableStopBits()),
+                                 QBindable<QSerialPort::StopBits>>);
 
-    sp.bindableStopBits().setBinding(Qt::makePropertyBinding(sbProp));
-    QCOMPARE(sp.stopBits(), QSerialPort::StopBits::OneAndHalfStop);
-
-    const QSignalSpy stopBitsChangedSpy(&sp, &QSerialPort::stopBitsChanged);
-
-    sbProp = QSerialPort::StopBits::OneStop;
-    QCOMPARE(sp.stopBits(), QSerialPort::StopBits::OneStop);
-    QCOMPARE(stopBitsChangedSpy.size(), 1);
-    QCOMPARE(stopBitsChangedSpy.at(0).value(0).toInt(), QSerialPort::StopBits::OneStop);
-
-    sbProp.setBinding(sp.bindableStopBits().makeBinding());
-    sp.setStopBits(QSerialPort::StopBits::TwoStop);
-    QCOMPARE(sbProp.value(), QSerialPort::StopBits::TwoStop);
-
-    sbProp.setBinding([&] { return sp.stopBits(); });
-    sp.setStopBits(QSerialPort::StopBits::OneAndHalfStop);
-    QCOMPARE(sbProp.value(), QSerialPort::StopBits::OneAndHalfStop);
+    QTestPrivate::testReadWritePropertyBasics(sp, QSerialPort::StopBits::OneAndHalfStop,
+                                              QSerialPort::StopBits::TwoStop, "stopBits");
+    if (QTest::currentTestFailed()) {
+        qDebug("Failed property test for QSetialPort::stopBits");
+        return;
+    }
 
     // -- flow control
 
-    QProperty<QSerialPort::FlowControl> fcProp(QSerialPort::FlowControl::HardwareControl);
-    QCOMPARE(fcProp.value(), QSerialPort::FlowControl::HardwareControl);
-
-    sp.bindableFlowControl().setBinding(Qt::makePropertyBinding(fcProp));
-    QCOMPARE(sp.flowControl(), QSerialPort::FlowControl::HardwareControl);
-
-    const QSignalSpy flowControlChangedSpy(&sp, &QSerialPort::flowControlChanged);
-
-    fcProp = QSerialPort::FlowControl::NoFlowControl;
-    QCOMPARE(sp.flowControl(), QSerialPort::FlowControl::NoFlowControl);
-    QCOMPARE(flowControlChangedSpy.size(), 1);
-    QCOMPARE(flowControlChangedSpy.at(0).value(0).toInt(), QSerialPort::FlowControl::NoFlowControl);
-
-    fcProp.setBinding(sp.bindableFlowControl().makeBinding());
-    sp.setFlowControl(QSerialPort::FlowControl::SoftwareControl);
-    QCOMPARE(fcProp.value(), QSerialPort::FlowControl::SoftwareControl);
-
-    fcProp.setBinding([&] { return sp.flowControl(); });
-    sp.setFlowControl(QSerialPort::FlowControl::NoFlowControl);
-    QCOMPARE(fcProp.value(), QSerialPort::FlowControl::NoFlowControl);
+    QTestPrivate::testReadWritePropertyBasics(sp, QSerialPort::FlowControl::HardwareControl,
+                                              QSerialPort::FlowControl::SoftwareControl,
+                                              "flowControl");
+    if (QTest::currentTestFailed()) {
+        qDebug("Failed property test for QSetialPort::flowControl");
+        return;
+    }
 
     // -- error
 
-    QProperty<QSerialPort::SerialPortError> errorProp(QSerialPort::SerialPortError::PermissionError);
-    QCOMPARE(errorProp.value(), QSerialPort::SerialPortError::PermissionError);
-
-    sp.bindableError().setBinding(Qt::makePropertyBinding(errorProp));
-    QCOMPARE(sp.error(), QSerialPort::SerialPortError::NoError);
-
-    const QSignalSpy errorChangedSpy(&sp, &QSerialPort::errorOccurred);
-
-    // this shall not change the error, we do not have a public setter
-    errorProp = QSerialPort::SerialPortError::DeviceNotFoundError;
-    QCOMPARE(sp.error(), QSerialPort::SerialPortError::NoError);
-    QCOMPARE(errorChangedSpy.size(), 0);
-
-    errorProp.setBinding(sp.bindableError().makeBinding());
-    sp.clearError();
-    QCOMPARE(errorProp.value(), QSerialPort::SerialPortError::NoError);
-    QCOMPARE(errorChangedSpy.size(), 1);
-    QCOMPARE(errorChangedSpy.at(0).value(0).toInt(), QSerialPort::SerialPortError::NoError);
-
-    sp.setPortName(m_receiverPortName);
-    sp.open(QIODevice::ReadOnly);
+    QTestPrivate::testReadOnlyPropertyBasics(
+            sp, QSerialPort::SerialPortError::NoError,
+            QSerialPort::SerialPortError::UnsupportedOperationError, "error",
+            [&sp]() { sp.open(QIODevice::Truncate); });
+    if (QTest::currentTestFailed()) {
+        qDebug("Failed property test for QSetialPort::error");
+        return;
+    }
 
     // -- break enabled
 
-    QProperty<bool> beProp(false);
-    QCOMPARE(beProp.value(), false);
+    sp.setPortName(m_receiverPortName);
+    const bool portOpened = sp.open(QIODevice::ReadOnly);
 
-    sp.bindableIsBreakEnabled().setBinding(Qt::makePropertyBinding(beProp));
-    QCOMPARE(sp.isBreakEnabled(), false);
+    if (portOpened) {
+        // To test the binding loop we need to create another instance of
+        // QSerialPort, open it, and set breakEnabled == true as a default
+        // value, so use a custom lambda for that.
+        QTestPrivate::testReadWritePropertyBasics<QSerialPort, bool>(
+                sp, true, false, "breakEnabled",
+                [name = this->m_senderPortName]() {
+                    auto ptr = std::make_unique<QSerialPort>();
+                    ptr->setPortName(name);
+                    if (ptr->open(QIODevice::ReadOnly)) {
+                        ptr->setBreakEnabled(true);
+                        return ptr;
+                    }
+                    // skip binding loop test if we failed to open the port
+                    return std::unique_ptr<QSerialPort>{};
+                });
+        if (QTest::currentTestFailed()) {
+            qDebug("Failed property test for QSetialPort::breakEnabled");
+            return;
+        }
+    } else {
+        // setting isBreakEnabled() will return false and raise an error
+        sp.setBreakEnabled(true);
+        QCOMPARE(sp.error(), QSerialPort::SerialPortError::NotOpenError);
+    }
+}
 
-    const QSignalSpy breakEnabledChangedSpy(&sp, &QSerialPort::breakEnabledChanged);
+// The reader connects to readyRead() and then calls waitForReadyRead()
+// inside the slot.
+// The tst_QSerialPort::readyReadNotRecursive() test needs to make sure
+// that the slot is entered only once.
+class SemiBlockingReader : public QObject
+{
+    Q_OBJECT
+public:
+    explicit SemiBlockingReader(const QString &portName)
+        : QObject(nullptr), m_portName(portName)
+    {}
 
-    beProp = true;
-    QCOMPARE(sp.isBreakEnabled(), true);
-    QCOMPARE(breakEnabledChangedSpy.size(), 1);
-    QCOMPARE(breakEnabledChangedSpy.at(0).value(0).toBool(), true);
+    int numSlotCalls() const { return m_numCalls; }
+    QByteArray receivedData() const { return m_data; }
 
-    beProp.setBinding(sp.bindableIsBreakEnabled().makeBinding());
-    sp.setBreakEnabled(false);
-    QCOMPARE(beProp.value(), false);
+public slots:
+    void start()
+    {
+        m_reader = new QSerialPort(m_portName, this);
+        connect(m_reader, &QSerialPort::readyRead, this, &SemiBlockingReader::readData);
+        if (!m_reader->open(QIODevice::ReadOnly))
+            emit error();
+    }
 
-    beProp.setBinding([&] { return sp.isBreakEnabled(); });
-    sp.setBreakEnabled(true);
-    QCOMPARE(beProp.value(), true);
+private slots:
+    void readData()
+    {
+        ++m_numCalls;
+        m_data.append(m_reader->readAll());
+        while (m_reader->waitForReadyRead(1000))
+            m_data.append(m_reader->readAll());
+        emit allDataReceived();
+    }
+
+signals:
+    void error();
+    void allDataReceived();
+
+private:
+    QString m_portName;
+    QByteArray m_data;
+    QSerialPort *m_reader = nullptr;
+    int m_numCalls = 0;
+};
+
+void tst_QSerialPort::readyReadNotRecursive()
+{
+    SemiBlockingReader reader(m_receiverPortName);
+    QThread readerThread;
+    connect(&readerThread, &QThread::started, &reader, &SemiBlockingReader::start);
+    connect(&reader, &SemiBlockingReader::allDataReceived, &readerThread, &QThread::quit);
+    int errorCount = 0;
+    connect(&reader, &SemiBlockingReader::error, this, [&] {
+        ++errorCount;
+        readerThread.quit();
+    });
+    bool readerThreadFinished = false;
+    connect(&readerThread, &QThread::finished, this, [&] { readerThreadFinished = true; });
+
+    reader.moveToThread(&readerThread);
+    readerThread.start();
+
+    QSerialPort senderPort(m_senderPortName);
+    QVERIFY(senderPort.open(QSerialPort::WriteOnly));
+    [[maybe_unused]] AsyncWriterByTimer writer(senderPort, Qt::DirectConnection,
+                                               alphabetArray, 50);
+
+    QTRY_VERIFY(readerThreadFinished);
+    QCOMPARE(errorCount, 0);
+    QCOMPARE(reader.numSlotCalls(), 1);
+    QCOMPARE(reader.receivedData(), alphabetArray);
 }
 
 QTEST_MAIN(tst_QSerialPort)
